/***************************************************************************
 *   Copyright (C) 2006 by Julien Lemoine, Simon Viennot                   *
 *   lapinot@tuxfamily.org                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef BOUNDARY_H
#define BOUNDARY_H

#include "representation.h"

typedef pair<Line,Line> couple;

//a Boundary has just a representation (it has no structure)

class Boundary : public Representation  {
  public:
	Boundary(const Line& init)  : Representation(init) {};
	
	//set to save Boundary::compute_oneBoundary_children() results and avoid useless recomputing
	set<couple> children_couple;
	
	//override terminal_vertex virtual function
	Vertex terminal_vertex();
	
	//----------- reduction/canonization ------------------------
	void sort();                 //sort algorithm (by successive rotations)
	void reverse_orientation();  //inversion of boundary's orientation
	bool isSingleSpot() const;   //return true if the boundary is "0."
	void distinguish_low_up();   //rename (if possible) upper-case vertices in lower-case ones
	void rename_low_in_order();  //rename lower-case vertices from the letter a
	
	//-----------boundary's children------------------------------
	void lower_to_upper();                  //rename all lower-case vertices in upper-case ones
	void compute_oneBoundary_children();    //children when the boundary is linked to itself
	void compute_twoBoundaries_children();    //children bribe when the boundary is linked to another one
};

#endif
