/***************************************************************************
 *   Copyright (C) 2006 by Julien Lemoine, Simon Viennot                   *
 *   lapinot@tuxfamily.org                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "game.h"

//Return "Win" or "Loss" if pnA check is correct
//Return "Computation_end" if check fails or if user stops computation
win_loss Game::check_win_loss(Pos_nim pnA){
	//Stop computation if asked by user, or if check computation failed.
	if (trace_computation.stop_signal()) {
		return global::Computation_end;
	}
	
	//Check if data already in check_base allow to compute the win/loss of pnA.
	Pos_nim pnA_simp=pnA;
	apply_known_nimbers(pnA_simp,false);	//"false" means "use check_base"
	
	//-----Manage to have only a single land to check------------------------------------------
	
	//If there are >=2 land, kill every land known in main_base
	if(Position(pnA_simp.first).position_type()==global::Multi_lands){
		if(kill_known_lands(pnA_simp)==false) return global::Computation_end;
	}
	
	//nimber is known for every land
	if(wl_is_known(pnA_simp) != global::Nim_unknown){
		check_graph.add_result(pnA_simp,wl_is_known(pnA_simp));
		check_graph.add_result(pnA,wl_is_known(pnA_simp));
		return wl_is_known(pnA_simp);
	}
	
	//-----Compute pnA_simp's children---------------------------------------------------------
	
	//create a link between a child, and this child simplified with "apply_known_nimbers" of check_base
	map<Pos_nim,Pos_nim> old_pn;
	
	list<Pos_nim> children_pos;		//list to stock position part children
	list<Pos_nim> children;		//list to stock all the children
	
	Pos_nim pn_temp;
	list<Pos_nim>::iterator PNi;
	map<Pos_nim,Pos_nim>::iterator pn_pn_i;
	
	//compute children
	pos_nim_children(pnA_simp, false, children_pos);	//"false" means children of position part
	
	for(PNi=children_pos.begin(); PNi!=children_pos.end(); PNi++) {
		cano(PNi->first);			//canonization
		pn_temp=(*PNi);
		apply_known_nimbers(*PNi,false);	//"false" means "use check_base"
		old_pn[pn_temp]=*PNi;			//link between old and new form of the child
		
		if(wl_is_known(*PNi)==global::Loss){
			check_graph.add_result(pnA_simp,global::Win);
			check_graph.add_result(pnA,global::Win);
			check_graph.add_result(pn_temp,global::Loss);
			check_graph.add_link(pnA_simp,pn_temp);
			return global::Win;	//returns "Win" if any child is "Loss"
		}
	}
	
	give_priority(children_pos);	//sort children
	children_pos.unique();	//removes all consecutive duplicate positions
	
	pos_nim_children(pnA_simp, true, children);		//"true" means we compute the children of nimber part
	children.insert(children.end(),children_pos.begin(),children_pos.end());	//gather all children
	
	//-----Use main_base to predict the result-------------------------------------------------
	
	win_loss wanted_result=global::Nim_unknown;	//result according to main_base (initialized to avoid warning)
	Pos_nim pn_simp_2=pnA_simp;
	list<Pos_nim> children_loss;			//list of pnA's losing children
	apply_known_nimbers(pn_simp_2,true);		//"true" means "use main_base"
	if(wl_is_known(pn_simp_2) == global::Loss){	//if pnA is "Loss", then pnA is stocked in main_base, so this test is ok.
		wanted_result=global::Loss;
	} else{						//pnA is "Win", or error
		for(PNi=children.begin(); PNi!=children.end(); PNi++) {
			pn_temp=*PNi;
			apply_known_nimbers(pn_temp,true);	//"true" means "use main_base"
			if(wl_is_known(pn_temp)==global::Loss){
				children_loss.push_back(*PNi);
				wanted_result=global::Win;
			}
		}
		if(children_loss.empty()){	//check failed : pnA isn't "Loss" and none of his children neither
			check_failure(pnA, 2);
			return global::Computation_end;
		}
	}
	
	//-----Check children----------------------------------------------------------------------
	
	win_loss children_expected_result=global::Win;
	
	//Preparation step when pnA is expected "Win"
	if(wanted_result==global::Win){
		//we need to check only the first expected losing children
		give_priority(children_loss);
		Pos_nim losing_child=children_loss.front();
		children.clear();
		children.push_back(losing_child);
		children_expected_result=global::Loss;
		
		//create a link with the form of the losing child before its simplification
		if(losing_child.first==pnA_simp.first){		//the losing child is a move in the nimber part
			check_graph.add_result(losing_child,global::Loss);
		} else{
			pn_pn_i=old_pn.begin();
			//find the form of the losing_child before simplification
			while(pn_pn_i->second != losing_child && pn_pn_i!=old_pn.end()){
				pn_pn_i++;
			}
			if(pn_pn_i==old_pn.end()) Error(QString("Element not found Game::check_win_loss"), true);
			check_graph.add_result(pn_pn_i->first,global::Loss);
			check_graph.add_link(pnA_simp,pn_pn_i->first);
		}
	} else{	//create a link with each child, with their form before simplification
		for(pn_pn_i=old_pn.begin();pn_pn_i!=old_pn.end();pn_pn_i++){
			check_graph.add_link(pnA_simp,pn_pn_i->first);
			check_graph.add_result(pn_pn_i->first,global::Win);
		}
	}
	
	//create a new level to trace computing (automatically destroyed when function returns)
	Level_trace one_level(&trace_computation, false);		//false : not a land computation
	trace_computation.add(children);
	
	//recursively check child(ren)
	int studied_pos_index=0;
	for(PNi=children.begin(); PNi!=children.end(); PNi++) {
		trace_computation.change_studied_pos(studied_pos_index);
		if(!check_expected_win_loss(*PNi, children_expected_result)) {
			return global::Computation_end;
		}
		trace_computation.known_child(studied_pos_index);
		studied_pos_index++;
	}
	
	//add position to database if it's a Loss
	//note that pnA is always constituted of a single land at this point of the function
	if(wanted_result==global::Loss) {
		add_to_database(pnA);
	}
	
	check_graph.add_result(pnA_simp,wanted_result);
	check_graph.add_result(pnA,wanted_result);
	
	return wanted_result;		//check ok
}

//check every land known in main_base for a position with several lands,
//return "false" in case of failure, "true" in case of success.
bool Game::kill_known_lands(Pos_nim& pnA) {
	list<Line> list_of_lands=Position(pnA.first).list_lands();
	list<Line>::iterator Li;	//Land i
	list<Line>::iterator Li_unknown;
	list<nimber> nimbers_known;
	int nb_unknown_land=0;
	map<string, nimber>::const_iterator SNi;
	Pos_nim one_land;		//land that needs to be checked
	
	for(Li=list_of_lands.begin(); Li!=list_of_lands.end(); Li++){
		SNi = main_base.find(toString(*Li));
		if(SNi != main_base.end()){
			//create a new level to trace computing (automatically destroyed at the end of the if)
			Level_trace one_level(&trace_computation, true);	//true: this is a land computation
			
			one_land.first=fromString(SNi->first);
			one_land.second=SNi->second;
			
			trace_computation.add(one_land);
			
			//recursive check of the land
			//we expect a Loss (it means the nimber in the main base is correct)
			if(!check_expected_win_loss(one_land, global::Loss)) {
				return false;
			}
			nimbers_known.push_back(SNi->second);	//check ok
		} else{	//if the land is unknown in main_base (shouldn't occur with more than 1 land).
			nb_unknown_land++;
			Li_unknown=Li;
		}
	}
	
	//error : too many unknown lands
	if(nb_unknown_land>=2){
		check_failure(pnA, 1);
		return false;
	}
	
	nimbers_known.push_back(pnA.second);	//add original nimber part
	pnA.second=nimber_xor(nimbers_known);	//new nimber computing
	
	if(nb_unknown_land==1){
		pnA.first=*Li_unknown;
	} else {//add empty position
		pnA.first.clear();
		pnA.first.push_back(global::End_Position);
	}
	
	return true;
}

//wrapping function of check_win_loss
//it returns true only if result is the expected one
bool Game::check_expected_win_loss(Pos_nim pnA, win_loss expected_win_loss) {
	win_loss result=check_win_loss(pnA);
	
	if(result==global::Computation_end){	//computation stopped
		return false;
	}
	
	if(result!=expected_win_loss) { //check failed
		check_failure(pnA, 3);
		return false;
	}
	
	//check is ok
	return true;
}

//complete check of a database : check all the nimbers in the main database
//returns Win in case of success, Loss if check fails
//there is a popup with a message when check fails
win_loss Game::complete_check() {
	map<string, nimber>::const_iterator PNi;
	
	//check all positions until it fails
	//already checked position are detected in check_win_loss
	for(PNi=main_base.begin(); PNi!=main_base.end(); PNi++) {
		Pos_nim pn;		//position that needs to be checked
		pn.first=fromString(PNi->first);
		pn.second=PNi->second;
		
		if(!check_expected_win_loss(pn, global::Loss)) {
			return global::Loss;
		}
	}
	
	//check succeeded
	return global::Win;
}

//sets the failure message according to the error that caused the check to fail
void Game::check_failure(Pos_nim pn, int failure_kind){
	check_failed=true;
	ostringstream oss;
	
	if (failure_kind==1) {
		oss << "This position have too many unknown land : ";
	} else if (failure_kind==2) {
		oss << "Main database isn't sufficient to determine this position's  Win/Loss : ";
	} else if (failure_kind==3) {
		oss << "This position should be X but is checked as not(X) : ";
	}
	
	oss << toString(pn.first) << " " << (int) pn.second;
	check_failure_msg=oss.str();
}
