/***************************************************************************
 *   Copyright (C) 2006 by Julien Lemoine, Simon Viennot                   *
 *   lapinot@tuxfamily.org                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef DOMAIN_H
#define DOMAIN_H

#include "boundary.h"

//Domain is a Structure of Boundary

class Domain : public Structure<Boundary>  {
  public:
	Domain(const Line& init) : Structure<Boundary>(init) {};
	
	//override terminal_vertex virtual function
	Vertex terminal_vertex();
	
	//----------reduction/canonization---------------------------------------------------------
	void simple_sort();            //only sort boundaries
	void sort();                   //sort boundaries (with possible inversion of orientation)
	bool dead() const;             //return true when domain is dead
	void merge_boundaries();       //merge little boundaries (theory of domain equivalence)
	
	//----------children-----------------------------------------------------------------------
	void compute_children();                //complete list of children
	void compute_oneBoundary_children();    //children when a boundary is linked to itself
	void compute_twoBoundaries_children();  //children when two different boundaries are linked
	
	set<couple> possible_partition;         //object to save boundaries repartition
	void partition(const list<Boundary>&);  //compute all ways to cut a list of boundaries in two parts
	
	//----------children number estimation-----------------------------------------------------
	unsigned int estimate_children_number();
	
	//----------lower/upper case letters dictinction -----------------------------------
	void distinguish_low_up();
	void rename_low_in_order();
	void lower_to_upper();
};

#endif
