/***************************************************************************
 *   Copyright (C) 2006 by Julien Lemoine, Simon Viennot                   *
 *   lapinot@tuxfamily.org                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//Constant values and functions used in the whole project

#include "global.h"


//-------------------------------------------------------------------------------------------------
// Global constant values
//-------------------------------------------------------------------------------------------------

//----------Special vertices values----------------------------------------------------------------

//"normal" vertex range from First_vtx to Last_vtx
//0, 1, 2 and 3 are reserved as generic vertices
//***WARNING*** : we should not use more than 26 upper-case or 26 lower-case letters without
//caring what happens in toString() and fromString()
const Vertex global::First_vtx=4;
const Vertex global::First_low_vtx=4;
const Vertex global::Last_low_vtx=29;
const Vertex global::First_up_vtx=30;
const Vertex global::Last_up_vtx=55;

//Child_vtx_1, Child_vtx_2, Child_vtx_3 are reserved letters used to name a new vertex
//they are used in Boundary::compute_oneBoundary_children() and Boundary::compute_twoBoundaries_children()
//Child_vtx_1, Child_vtx_2 and Child_vtx_3 are nonetheless "normal" vertices (between First_vtx and Last_vtx)
const Vertex global::Child_vtx_1=56;
const Vertex global::Child_vtx_2=57;
const Vertex global::Child_vtx_3=58;
const Vertex global::Last_vtx=58;

//Special vertices indicating the end of a structure in a vertices list representation
const Vertex global::End_Boundary=59;
const Vertex global::End_Domain=60;
const Vertex global::End_Land=61;
const Vertex global::End_Position=62;


//----------constant values to indicate the kind of position---------------------------------------
//used as return values of Position::position_type()

//empty position
const char global::Empty_position=1;

//position is constituted of a single land
const char global::Single_land=2;

//position is constituted of two or more lands
const char global::Multi_lands=3;


//----------graphical interface--------------------------------------------------------------------
//maximum number of displayed level
const int global::Max_Level=25;


//----------nimber and win_loss--------------------------------------------------------------------
//maximal value of a valid nimber
const nimber global::Nim_max=253;

//unknown nimber
const nimber global::Nim_unknown=254;

//nimber computation was interrupted
const nimber global::Computation_end=255;

//win_loss
const win_loss global::Loss=0;
const win_loss global::Win=1;


//----------computation or database kind-----------------------------------------------------------
const int global::Main=0;
const int global::Check=1;
const int global::Complete_check=2;
const int global::Analysis=3;
const int global::Graph=4;


//-------------------------------------------------------------------------------------------------
// Line <---> string conversion
//-------------------------------------------------------------------------------------------------

//special characters used in string
#define Char_End_Boundary '.'
#define Char_End_Domain '}'
#define Char_End_Land ']'
#define Char_End_Position '!'

//convert a list of vertices in a string
string toString(const Line& r_vtx) {
	ostringstream output;
	Line::const_iterator s;
	
	for(s=r_vtx.begin(); s!=r_vtx.end(); s++) {
		switch(*s) {
			case global::End_Boundary : output << Char_End_Boundary; break;
			case global::End_Domain   : output << Char_End_Domain;   break;
			case global::End_Land     : output << Char_End_Land;     break;
			case global::End_Position : output << Char_End_Position; break;
			case 0 : output << '0'; break;
			case 1 : output << '1'; break;
			case 2 : output << '2'; break;
			case 3 : output << '3'; break;
			default:
				//lower case letters
			if (*s>=global::First_low_vtx && *s<=global::Last_low_vtx) {
				//First_low_vtx->a, First_low_vtx+1->b, etc
				output << (unsigned char) (*s-global::First_low_vtx+97);
			} else {
				//upper case letters
				if (*s>=global::First_up_vtx && *s<=global::First_up_vtx+25) {
					//First_up_vtx->A, First_up_vtx+1->B, etc
					output << (unsigned char) (*s-global::First_up_vtx+65);
				} else {//in case of more than 26 upper case letters
					//First_up_vtx+26 ->car 128
					output << (unsigned char) (*s-global::First_up_vtx-26+128);
				}
			}
		}
	}
	
	return output.str();
}

//convert string in a list of vertex
Line fromString(const string& stringA) {
	Line r_vtx;		//returned vertices list
	int i;
	unsigned char c;
	Vertex val;
	
	int n=stringA.length();
	
	for(i=0;i<n;i++) {
		c=stringA[i];
		switch(c) {
			case Char_End_Boundary : val = global::End_Boundary; break;
			case Char_End_Domain   : val = global::End_Domain;   break;
			case Char_End_Land     : val = global::End_Land;     break;
			case Char_End_Position : val = global::End_Position; break;
			case '0' : val = 0; break;
			case '1' : val = 1; break;
			case '2' : val = 2; break;
			case '3' : val = 3; break;
			default:
			if(c>=97 && c < 128) {
				//a->First_low_vtx, b->First_low_vtx+1, etc.
				val = (unsigned char) (c-97+global::First_low_vtx);
			} else {
				//test used to treat more than 26 letters
				if(c<128) {
					//A->First_up_vtx, B->First_up_vtx+1, etc
					val = (unsigned char) (c-65+global::First_up_vtx);
				} else {
					//128->First_up_vtx+26, etc
					val = (unsigned char) (c-128+global::First_up_vtx+26);
				}
			}
		}
		r_vtx.push_back(val);
	}
	
	return r_vtx;
}


//-------------------------------------------------------------------------------------------------
// Number of lives
//-------------------------------------------------------------------------------------------------

//Number of lives of a vertices list
//input list is supposed reduced : all letters appear twice and refer to vertex of degree 2 (one life)
int nb_lives(const Line& r_vtx) {
	int lives=0;
	int letters=0;
	
	Line::const_iterator s;
	
	for(s=r_vtx.begin(); s!=r_vtx.end(); s++) {
		if(*s>=global::First_vtx && *s<=global::Last_vtx) {
			letters++;
		} else
		
		if(*s==2) {
			lives+=1;
		} else
		
		if(*s==1) {
			lives+=2;
		} else
		
		if(*s==0) {
			lives+=3;
		}
	}
	
	//add half the number of letters to the number of lives
	lives += letters/2;
	
	return lives;
}


//-------------------------------------------------------------------------------------------------
// Handle vertices
//-------------------------------------------------------------------------------------------------

//Test if a vertex is upper case (not a temporary one)
bool is_upper_case(const Vertex& s) {
	return (s>=global::First_up_vtx && s<=global::Last_up_vtx);
}

//Test if a vertex is lower case
bool is_lower_case(const Vertex& s) {
	return (s>=global::First_low_vtx && s<=global::Last_low_vtx);
}

//Test if a vertex is a temporary one
bool is_temp_vtx(const Vertex& s) {
	return (s>=global::Child_vtx_1 && s<=global::Child_vtx_3);
}

//Test if a vertex is a letter (not generic or End*)
bool is_letter_vtx(const Vertex& s) {
	return (s>=global::First_vtx && s<=global::Last_vtx);
}

//Test if a vertex is a real one (generic, one_life, temp : ok, but not End_* vertices)
bool is_real_vtx(const Vertex& s) {
	return (s<=global::Last_vtx);
}

//test if there is a lower-case Vertex in a Line
bool exist_lower(const Line& r_vtx) {
	bool result=false;
	Line::const_iterator s;
	
	for(s=r_vtx.begin(); s!=r_vtx.end() && !result; s++) {
		result=is_lower_case(*s);
	}
	
	return result;
}


//-------------------------------------------------------------------------------------------------
// Other functions
//-------------------------------------------------------------------------------------------------

//power of 2 (math function used in Domain::partition and Domain::estimate_children_number)
// important note : this function is false when n is too big, but this is not always a problem
// we use a parameter to distinguish between the two cases.
// overflow needs treatment in Domain::partition because it could cause false children computations
// overflow is not a problem in Domain::estimate_children_number because we only compute an estimation (it doesn't need accuracy)
unsigned long powers_of_two(int n, bool treatErrors) {
	unsigned long result=1;
	for(int i=0; i<n; i++) {
		if (treatErrors && result*2<result) {
			Error(QString("powers_of_two : too big number to compute the powers of 2"), true);
		}
		
		result=result*2;
	}
	
	return result;
}
