/***************************************************************************
 *   Copyright (C) 2006 by Julien Lemoine, Simon Viennot                   *
 *   lapinot@tuxfamily.org                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//global types and constant values (used in the whole project)

#ifndef GLOBAL_H
#define GLOBAL_H

//the following files are included in the whole project
#include <list>
#include <set>
#include <map>
#include <vector>

#include <iostream>
#include <sstream>

using namespace std;

//error handling
#include "error.h"

//vertex type (unsigned char between 0 and 255)
typedef unsigned char Vertex;

//line type : representation of any sprouts object with a list of vertex
typedef list<Vertex> Line;

//nimber type
typedef unsigned char nimber;

//win-loss type
typedef nimber win_loss;

//extended position type : (position, nimber)
typedef pair<Line, nimber> Pos_nim;

//conversions between string and Line types
string toString(const Line& r_vtx);
Line fromString(const string& stringA);

//number of lives of a vertices list
int nb_lives(const Line& r_vtx);

//powers of 2 (with or without overflow handling)
unsigned long powers_of_two(int n, bool treatErrors);

//handle vertices
bool is_upper_case(const Vertex&);
bool is_lower_case(const Vertex&);
bool is_temp_vtx(const Vertex&);
bool is_letter_vtx(const Vertex&);
bool is_real_vtx(const Vertex&);
bool exist_lower(const Line&);

//global constant values (we prefer class static members instead of #define)
class global {
  public :
	//special vertices values (mostly used through the above functions "handle vertices")
	static const Vertex First_vtx, First_low_vtx, Last_low_vtx, First_up_vtx, Last_up_vtx;
	static const Vertex Child_vtx_1, Child_vtx_2, Child_vtx_3, Last_vtx;
	static const Vertex End_Boundary, End_Domain, End_Land, End_Position;
	
	//position kind
	static const char Empty_position, Single_land, Multi_lands;
	
	//maximum number of levels displayed in the interface
	static const int Max_Level;
	
	//nimber and win_loss special values
	static const nimber Nim_max, Nim_unknown, Computation_end;
	static const win_loss Loss, Win;
	
	//computation or database kind
	static const int Main, Check, Complete_check, Analysis, Graph;
};

#endif
