/***************************************************************************
 *   Copyright (C) 2006 by Julien Lemoine, Simon Viennot                   *
 *   lapinot@tuxfamily.org                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef GLOPWIDGET_H
#define GLOPWIDGET_H

#include <QMainWindow>
#include <QWidget>
#include "game.h"
#include "tracewidget.h"

//graphical user interface classes
class QTimer;
class QLabel;
class QString;
class QTabWidget;
class QSpinBox;
class QCheckBox;
class QRadioButton;
class QSignalMapper;
class QLineEdit;
class QTextEdit;
class QCloseEvent;

//class for the main window
class GlopWidget : public QMainWindow {
	Q_OBJECT
  public:
	GlopWidget( QWidget *parent=0 );
  //main object representing the game
	Game *game;
	
  //graphical interface
	//database size
	QLabel *mainSizeLabel, *checkSizeLabel, *graphSizeLabel, *analysisSizeLabel;
	
	//tabs
	QTabWidget *tabWidget;
	void setup_parameters_tab();
	void setup_compute_tab();
	void setup_analysis_tab();
	void setup_test_tab();
	void setup_info_tab();
	
	//widget parameters
	QSpinBox *box_spots_number, *box_given_nimber, *box_refresh, *box_detail, *box_mlg;
	QCheckBox *box_use_colors, *box_use_graph;
	QRadioButton *classic_position, *given_position;
	QLineEdit *box_given_position, *box_given_position_2;
	
	//analysis parameters widget
	QCheckBox *check_lives_max, *check_lives_min, *check_nimber_max, *check_nimber_min;
	QSpinBox *box_lives_max, *box_lives_min, *box_nimber_max, *box_nimber_min;
	
	//connexion mapping of fileSaveAs function
	QSignalMapper *fileSaveAs_Mapper, *startCompute_Mapper, *purgeBase_Mapper, *readText_Mapper;
	
	//tables to trace computation
	WidgetTrace *widget_trace;
	
	//to print children
	QTextEdit *textEdit_children;
	
	void refresh_base_size(int num_base);	//refresh base size
	
  //files read/write functions
	void write_file( const QString &filename , int num_base);
	void read_file( const QString &filename );
	
  protected:
	//override QApplication virtual function closeEvent
	void closeEvent(QCloseEvent *event);
	
	//parameters to save the base state
	unsigned int mainbasesize, checkbasesize;
	bool mainbaseisSaved, checkbaseisSaved, computeRunning;
	
  public slots:
	//menu
	void read_text_file(int num);

	//start/stop computation
	void start_computation(int computation_kind);
	void read_computation_parameters();
	void read_graph_parameters();
	void read_analysis_parameters();
	void stop_computing();
	
  //trace computation related functions
	void activate_timer(bool);
	void trace_refresh(); //called on each timer's impulse
	void wait_result();
	
  //test functions
	void print_children();
	
  //read/write bases
	void purgeBase(int num_base);
	void fileSaveAs(int num_base);
	void fileSaveGraph();
	void fileOpen();
	
  signals:
	void isComputing(bool);	//specifies that the computation starts/stops
	void isChecking(bool);	//check computing only
	
  private:
	QTimer * impulse; //timer object with regular impulsion to refresh the graphical interface
};

#endif
