/***************************************************************************
 *   Copyright (C) 2006 by Julien Lemoine, Simon Viennot                   *
 *   lapinot@tuxfamily.org                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef GRAPH_H
#define GRAPH_H

#include "global.h"
#include "position.h"

#include <QMutexLocker>

//this is just an arbitrary number that corresponds to a single position.
typedef unsigned long pos_index;

//win_loss describes the status of the position stored in the string :
// - if it's "Win", we still don't know the position's nimber, so we store in the set all the nimbers
//known to be wrong.
// - if it's "Loss", we know the position's nimber, and we store it in the set, so that the set contains
//only one nimber.
//During a check computation, it is possible to have a position with status "Win",
//then with status "Loss" later.
typedef pair<win_loss,set<nimber> > nimber_info;

class QString;
class QMutex;

//This class is used to produce files directly usable in Graphviz
//It is a lot similar to Trace
class Graph {
  private:
	//---------- databases -------------------------------------------
	map<string, nimber_info > nimbers_base;
	map<string, pos_index> index_base;
	
	//for each pos_index, store the list of children pos_index
	map<pos_index, set<pos_index> > link_base;
	
	//---------- internal functions ----------------------------------
	pos_index get_index(const Pos_nim& p);		//get existing index or create new one
	map<string, pos_index>::iterator find_label(pos_index index);	//find a label with its index in index_base
	void simplify_link_base();			//delete multiple links from a position to children with same nimbers
	QString print_position(pos_index);		//print the position of the given index
	
  public:
	//lock used only because we use the function size() of the different bases in the graphical interface
	QMutex graph_lock;
	
	//---------- interface -------------------------------------------
	void initialize();				//clear graph databases
	void add_result(const Pos_nim&, win_loss);	//update nimbers_base's informations with Pos_nim's data
	void add_link(const Pos_nim&, const Pos_nim&);	//add a link between parent and child in the link database
	void write_file( const QString &filename);	//write a file with "dot" format (for Graphviz)
	unsigned long get_size();
	
	//---------- parameters ------------------------------------------
	bool use_graph;		//to activate the graph
	bool use_colors;	//to draw a graph with colors
	int detail_level;	//detail=1:node=dot; 2:node=number; 3:node=position+nimber_info
	int min_lives;		//a position with less lives than this number won't appear in the graph
};

#endif
