/***************************************************************************
 *   Copyright (C) 2006 by Julien Lemoine, Simon Viennot                   *
 *   lapinot@tuxfamily.org                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "tracewidget.h"
#include <QtGui>

//--------------------------------------------------------------------------
//Widgets : tables to display computing status
//--------------------------------------------------------------------------

//initialization of main trace table
WidgetTrace::WidgetTrace(QWidget *parent, Trace* trace_source)
 : QWidget( parent) {
	//save pointer to the trace object
	trace=trace_source;
	
	QVBoxLayout *vboxLayout = new QVBoxLayout(this);
	vboxLayout->setMargin(0);
	table=new QTableWidget(global::Max_Level, 5, this);
	table->setEnabled(false);
	
	QStringList titles;
	titles << "Index  " << "Alive" << "Lives" << "Nimber" << "Position";
	table->setHorizontalHeaderLabels(titles);
	
	int i, j;
	//initialization of columns width
	for(j=0; j<4; j++) {
		table->resizeColumnToContents(j);
	}
	table->setColumnWidth(4,500);
	//initialization with empty items
	for(i=0; i<global::Max_Level; i++) {
		for(j=0; j<5; j++) {
			QTableWidgetItem *item = new QTableWidgetItem(tr("%1").arg( (i+1)*(j+1) ));
			item->setText("");
			table->setItem(i, j, item);
		}
	}
	
	//no edit of the table allowed
	table->setEditTriggers ( QAbstractItemView::NoEditTriggers );
	
	//add the table to the layout
	vboxLayout->addWidget(table);
	
	//click on the table
	connect( table, SIGNAL( itemClicked(QTableWidgetItem*) ), this, SLOT( click_on_position(QTableWidgetItem*) ) );
	
	//table to trace children
	children_popup=new WidgetChildren(this, trace);
}

//initialization of children trace table
WidgetChildren::WidgetChildren(QWidget *parent, Trace* trace_source)
 : QDialog( parent) {
	//save pointer to the trace object
	trace=trace_source;
	
	setMinimumWidth( 700 );
	setMinimumHeight( 500 );
	gridLayout = new QGridLayout(this);
	
	//display parent
	QLabel *title_parent = new QLabel(this);
	title_parent->setText("Parent :");
	
	parent_position = new QLineEdit(this);
	parent_position->setMinimumWidth( 150 );
	parent_position->setReadOnly( true );
	
	QLabel *title_nimber = new QLabel(this);
	title_nimber->setText("Nimber :");
	
	nimber_parent = new QLineEdit(this);
	nimber_parent->setFixedWidth( 50 );
	nimber_parent->setReadOnly( true );
	
	//add the widgets to the layout
	gridLayout->addWidget( title_parent, 0, 0 );
	gridLayout->addWidget( parent_position, 0, 1 );
	gridLayout->addWidget( title_nimber, 0, 2 );
	gridLayout->addWidget( nimber_parent, 0, 3 );
	
	//indicates that children's table have never been created/deleted
	first_time=true;
}

//create a new table for children, depending on the number of those children
void WidgetChildren::create_table(int nb_rows) {
	table=new QTableWidget(nb_rows, 3, this);
	
	QStringList titles;
	titles << "Lives" << "Nimber" << "Position";
	table->setHorizontalHeaderLabels(titles);
	
	int i, j;
	//column width initialization
	for(j=0; j<2; j++) {
		table->resizeColumnToContents(j);
	}
	table->setColumnWidth(2,500);
	//initialization with empty items
	for(i=0; i<nb_rows; i++) {
		for(j=0; j<3; j++) {
			QTableWidgetItem *item = new QTableWidgetItem(tr("%1").arg( (i+1)*(j+1) ));
			item->setText("");
			table->setItem(i, j, item);
		}
	}
	
	//table is not editable
	table->setEditTriggers ( QAbstractItemView::NoEditTriggers );
	
	gridLayout->addWidget(table, 1, 0, 1, 4);	//children table uses 1 and 4 columns of the layout
}

//delete children's table
void WidgetChildren::delete_table() {
	delete table;
}


//--------------------------------------------------------------------------
//Widgets : refresh functions
//--------------------------------------------------------------------------

//refresh tracing table
void WidgetTrace::update_display() {
	QMutexLocker locker(&(trace->trace_lock));
	//security to avoid possible crashs
	if (trace->traced_levels.empty() ) {
		return;
	}
	
	list< Level >::const_iterator Li;
	list< PositionTrace >::const_iterator PTi;
	QTableWidgetItem * item;
	
	int i,row;
	QString item_string;
	
	for(row=0, Li=trace->traced_levels.begin(); row<global::Max_Level; row++) {
		//the second test (security) should stay there as long as we haven't found a better way of programming
		//(ie add should be called in the same lock than create_level).
		if(Li!=trace->traced_levels.end() && !(Li->traced_positions).empty()) {
			//PTi is placed on the studied position
			PTi=(Li->traced_positions).begin();
			for(i=0;i< Li->studied_pos_index;i++){
				PTi++;
			}
			
			//progress
			item_string=QString( "%1 " ).arg( Li->studied_pos_index +1, 2, 10 );
			item_string+=QString("/");
			item_string+=QString( "%1 " ).arg(Li->traced_positions.size(), 2, 10 );
			item=table->item(row,0);
			item->setData(Qt::DisplayRole, item_string);
			
			//number of still alive positions
			item_string=QString( "%1 " ).arg(Li->nb_alive_pos, 2, 10 );
			item=table->item(row,1);
			item->setData(Qt::DisplayRole, item_string);
			if(Li->nb_alive_pos==1){
				item->setBackgroundColor( QColor(255,127,127) );	//red
			} else if (Li->nb_alive_pos<5) {
				item->setBackgroundColor( QColor(127,255,127) );	//green
			} else {
				item->setBackgroundColor( QColor(255,255,255,0) );	//alpha=0 : no color
			}
			
			//lives
			item_string=QString( "%1 " ).arg(PTi->nb_lives, 2, 10 );
			item=table->item(row,2);
			item->setData(Qt::DisplayRole, item_string);
			
			//nimber
			item_string=QString( "%1 " ).arg((int) (PTi->nim), 2, 10 );
			item=table->item(row,3);
			item->setData(Qt::DisplayRole, item_string);
			
			//position
			item_string=QString::fromStdString( toString(PTi->r_vtx) );
			item=table->item(row,4);
			item->setData(Qt::DisplayRole, item_string);
			if(Li->nimber_level){
				item->setBackgroundColor( QColor(255,255,127) );	//yellow
			} else{
				item->setBackgroundColor( QColor(255,255,255,0) );	//alpha=0 : no color
			}
			
			//iterator incrementation
			Li++;
		} else {
			//there's nothing to display (end of list or level not yet filled with positions)
			item=table->item(row,0);
			item->setData(Qt::DisplayRole, "");
			item=table->item(row,1);
			item->setBackgroundColor( QColor(255,255,255,0) );	//alpha=0 : no color
			item->setData(Qt::DisplayRole, "");
			item=table->item(row,2);
			item->setData(Qt::DisplayRole, "");
			item=table->item(row,3);
			item->setData(Qt::DisplayRole, "");
			item=table->item(row,4);
			item->setData(Qt::DisplayRole, "");
			item->setBackgroundColor( QColor(255,255,255,0) );	//alpha=0 : no color
		}
	}
	//----------------children table--------------------------------
	//security : there is no table to print if parent_level==-1
	if(trace->parent_level!=-1) {
		children_popup->update_display();
	} else {
	//close children dialog
		children_popup->close();
	}
}

//refresh displayed children table
void WidgetChildren::update_display() {
	//note : no lock, this function is called only from an already locked function
	
	list< Level >::const_iterator Li=trace->traced_levels.begin(); //level list iterator
	list< PositionTrace >::const_iterator PTi;
	
	QTableWidgetItem * item;
	int i;
	QString item_string;
	
	//Li is placed on the parent's level
	for(i=1;i<trace->parent_level;i++){
		Li++;
	}
	
	//display parent
	if ( !( (Li->traced_positions).empty() ) ) {
		//find the PositionTrace of the parent
		PTi=(Li->traced_positions).begin();
		for(i=0;i< Li->studied_pos_index;i++){
			PTi++;
		}
		//display position and nimber
		item_string=QString::fromStdString( toString(PTi->r_vtx) );
		parent_position->setText(item_string);
		item_string=QString( "%1 " ).arg((int) (PTi->nim), 2, 10 );
		nimber_parent->setText(item_string);
	}
	
	//display children
	Li++;
	int row=0;
	
	for(PTi=(Li->traced_positions).begin();PTi!=(Li->traced_positions).end();PTi++){
		//lives
		item_string=QString( "%1 " ).arg(PTi->nb_lives, 2, 10 );
		item=table->item(row,0);
		item->setData(Qt::DisplayRole, item_string);
		//nimber
		item_string=QString( "%1 " ).arg((int) (PTi->nim), 2, 10 );
		item=table->item(row,1);
		item->setData(Qt::DisplayRole, item_string);
		//position
		item_string=QString::fromStdString( toString(PTi->r_vtx) );
		item=table->item(row,2);
		item->setData(Qt::DisplayRole, item_string);
		//cell colored in gray when the child is dead
		if(PTi->status_unknown){
			item->setBackgroundColor( QColor(255,255,255,0) );	//alpha=0 : no color
		}
		else{
			item->setBackgroundColor( QColor(127,127,127) );	//gray
		}
		//currently studied child colored in blue
		if(row==Li->studied_pos_index){
			item->setBackgroundColor( QColor(127,127,255) );	//blue
		}
		row++;
	}
}

//Function called after a click in the trace widget
//click on left column : zap child
//click on the position cell : display children
void WidgetTrace::click_on_position( QTableWidgetItem* item ) {
	if ( !item ) return;
	
	table->setItemSelected(item, FALSE );
	
	//-----click on a left column-----------------
	//go to the next or the previous branch of the level
	if (table->column(item)<4) {
		trace->request_level( table->row(item) +1 , table->column(item)==3 );
		return;
	}
	
	//-----click on a right column----------------
	QMutexLocker locker(&(trace->trace_lock));
	
	int parent_candidate_level=(table->row(item) +1);
	
	//we do nothing if the click was too low in the interface
	if(parent_candidate_level >= trace->current_level -1) return;
	
	//a position cell was clicked, we display children
	//the clicked position becomes the parent of children table
	//clicked position's line gives table index
	
	//update trace informations : new parent for children's widget
	trace->parent_level=parent_candidate_level;
	int children_number=trace->level_size(parent_candidate_level);
	
	//delete old table (if existing) before creating a new one with the right size
	if(children_popup->first_time) {
		children_popup->first_time=false;
	}
	else{
		disconnect( children_popup->table, SIGNAL( itemClicked(QTableWidgetItem*) ),
		            children_popup       , SLOT( click_on_children(QTableWidgetItem*) ) );
		children_popup->delete_table();
	}
	
	children_popup->create_table(children_number);
	
	//connect the new table to activate clicks on children
	connect( children_popup->table, SIGNAL( itemClicked(QTableWidgetItem*) ),
	         children_popup       , SLOT( click_on_children(QTableWidgetItem*) ) );
	
	children_popup->update_display();
	children_popup->show();
	children_popup->raise();
}

//function called when the children's table is clicked
void WidgetChildren::click_on_children( QTableWidgetItem* item ) {
	if ( !item ) return;
	
	table->setItemSelected(item, FALSE );
	
	//request trace to go to clicked child
	trace->request_child( table->row(item) );
}
